import { Component, createElement } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

const classNames = (obj) => {
    return Object.keys(obj)
        .reduce((acc, name) => (obj[name] ? [...acc, name] : acc), [])
        .join(' ');
};
const range = (length) => Array(length)
    .fill(null)
    .map((_, i) => i);
const sameDays = (d1, d2) => {
    return d1.getFullYear() === d2.getFullYear() && d1.getMonth() === d2.getMonth() && d1.getDate() === d2.getDate();
};
const deltaDate = (date, yearDelta, monthDelta = 0, dayDelta = 0) => {
    return new Date(date.getFullYear() + yearDelta, date.getMonth() + monthDelta, date.getDate() + dayDelta);
};
const getFirstDayOfMonth = (date) => new Date(date.getFullYear(), date.getMonth(), 1);
const getFirstDayOfWeek = (date, weekStart) => {
    const diff = (date.getDay() < weekStart ? 7 : 0) + date.getDay() - weekStart;
    return new Date(date.getFullYear(), date.getMonth(), date.getDate() - diff);
};
const getLastDayOfMonth = (date) => new Date(date.getFullYear(), date.getMonth() + 1, 0);
const getLastDayOfWeek = (date, weekStart) => {
    const diff = (date.getDay() < weekStart ? -7 : 0) + 6 - (date.getDay() - weekStart);
    return new Date(date.getFullYear(), date.getMonth(), date.getDay() + diff);
};
const generateMatrix = (date, weekStart) => {
    const firstDayOfMonth = getFirstDayOfMonth(date);
    const startDate = getFirstDayOfWeek(firstDayOfMonth, weekStart);
    return range(6).map((_, i) => {
        return range(7).map((__, j) => {
            return new Date(startDate.getFullYear(), startDate.getMonth(), startDate.getDate() + i * 7 + j);
        });
    });
};
const isInMatrix = (matrix, date) => {
    return matrix.some(row => {
        return row.some(rowDate => sameDays(rowDate, date));
    });
};
const abbreviationDays = (abbreviations, weekStart) => {
    return range(7).map((_, index) => {
        const dayIndex = index + weekStart;
        return dayIndex >= 7 ? abbreviations[dayIndex - 7] : abbreviations[dayIndex];
    });
};

const Abbreviations = (_a) => {
    var { i18n, weekStart } = _a, props = __rest(_a, ["i18n", "weekStart"]);
    return (createElement("tr", Object.assign({ className: "weekdays" }, props), abbreviationDays(i18n.weekdaysShort, weekStart).map((day) => (createElement("td", { key: day, colSpan: 1 }, day)))));
};

const Day = (_a) => {
    var { day, value, screen, cursor } = _a, props = __rest(_a, ["day", "value", "screen", "cursor"]);
    const isPrevMonth = screen.getMonth() === 0 ? day.getMonth() === 11 : day.getMonth() === screen.getMonth() - 1;
    const isNextMonth = screen.getMonth() === 11 ? day.getMonth() === 0 : day.getMonth() === screen.getMonth() + 1;
    const isInThisMonth = !isPrevMonth && !isNextMonth;
    const classNamesString = classNames({
        day: true,
        grayed: !isInThisMonth,
        selected: sameDays(day, value),
        cursor: sameDays(day, cursor),
        today: sameDays(day, new Date())
    });
    return (createElement("td", Object.assign({}, props, { className: classNamesString }), day.getDate()));
};

const yearOption = (year) => {
    return (createElement("option", { key: year, value: year }, year));
};
const Header = (_a) => {
    var { i18n, screen, onScreenChange } = _a, props = __rest(_a, ["i18n", "screen", "onScreenChange"]);
    const screenYear = screen.getFullYear();
    const yearRange = range(8);
    const minusYears = yearRange.map(value => yearOption(screenYear - yearRange.length + value));
    const plusYears = yearRange.map(value => yearOption(screenYear + value + 1));
    const yearOptions = [...minusYears, yearOption(screenYear), ...plusYears];
    return (createElement("tr", Object.assign({}, props),
        createElement("th", { colSpan: 1 },
            createElement("a", { className: "chevron", onClick: () => onScreenChange(deltaDate(screen, 0, -1)) }, "\u2039")),
        createElement("th", { colSpan: 5 },
            createElement("select", { value: screen.getMonth(), onChange: e => onScreenChange(new Date(screen.getFullYear(), parseInt(e.target.value, 10), screen.getDate())) }, i18n.months.map((month, index) => (createElement("option", { key: month, value: index }, month)))),
            createElement("select", { value: screen.getFullYear(), onChange: e => onScreenChange(new Date(parseInt(e.target.value, 10), screen.getMonth(), screen.getDate())) }, yearOptions)),
        createElement("th", { colSpan: 1 },
            createElement("a", { className: "chevron", onClick: () => onScreenChange(deltaDate(screen, 0, 1)) }, "\u203A"))));
};

const DatePicker = (_a) => {
    var { matrix, renderDay, header, abbreviations, onKeyDown } = _a, props = __rest(_a, ["matrix", "renderDay", "header", "abbreviations", "onKeyDown"]);
    return (createElement("table", Object.assign({ className: "fus-datepicker", tabIndex: 0, onKeyDown: onKeyDown }, props),
        createElement("thead", null,
            header,
            abbreviations),
        createElement("tbody", null, matrix.map(row => createElement("tr", { key: 'row' + row[0].toString() }, row.map(date => renderDay(date)))))));
};

const i18n = {
    previousMonth: 'Previous month',
    nextMonth: 'Next month',
    months: [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December'
    ],
    weekdays: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
    weekdaysShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']
};

function defaultOnKeyDown(e, matrix, onChange, onScreenChange, onCursorChange, cursor, screen) {
    if (e.key === 'enter') {
        onChange(cursor);
        return;
    }
    const cursorIsInMatrix = isInMatrix(matrix, cursor);
    let newCursor = null;
    switch (e.key) {
        case 'ArrowDown': {
            newCursor = cursorIsInMatrix ? deltaDate(cursor, 0, 0, 7) : getFirstDayOfMonth(screen);
            break;
        }
        case 'ArrowUp': {
            newCursor = cursorIsInMatrix ? deltaDate(cursor, 0, 0, -7) : getLastDayOfMonth(screen);
            break;
        }
        case 'ArrowLeft': {
            newCursor = cursorIsInMatrix ? deltaDate(cursor, 0, 0, -1) : getLastDayOfMonth(screen);
            break;
        }
        case 'ArrowRight': {
            newCursor = cursorIsInMatrix ? deltaDate(cursor, 0, 0, 1) : getFirstDayOfMonth(screen);
            break;
        }
        case 'Enter': {
            onChange(cursor);
            break;
        }
    }
    if (newCursor) {
        onCursorChange(newCursor);
        onScreenChange(newCursor);
    }
}
class SimpleDatePicker extends Component {
    constructor() {
        super(...arguments);
        this.state = {
            cursor: new Date(this.props.value),
            screen: new Date(this.props.value)
        };
        this.onChange = (value) => {
            this.setState({
                screen: value
            });
            this.props.onChange(value);
        };
        this.onCursorChange = (cursor) => {
            this.setState({ cursor });
        };
        this.onScreenChange = (screen) => {
            this.setState({ screen });
        };
        this.onKeyDown = (keyDownEvent, matrix) => {
            defaultOnKeyDown(keyDownEvent, matrix, this.onChange, this.onScreenChange, this.onCursorChange, this.state.cursor, this.state.screen);
        };
        this.renderDay = (day, value, screen, cursor, onClick, onMouseMove) => {
            return (createElement(Day, { key: day.toString(), day: day, value: value, screen: screen, cursor: cursor, onClick: onClick, onMouseMove: onMouseMove }));
        };
        this.renderAbbreviations = (i18n$$1, weekStart) => {
            return createElement(Abbreviations, { i18n: i18n$$1, weekStart: weekStart });
        };
        this.renderHeader = (i18n$$1, screen, onScreenChange) => {
            return createElement(Header, { i18n: i18n$$1, screen: screen, onScreenChange: onScreenChange });
        };
    }
    render() {
        const weekStart = this.props.weekStart || 0;
        const i18n$$1 = this.props.i18n || i18n;
        const matrix = generateMatrix(this.state.screen, weekStart);
        const renderDay = (day) => (this.props.renderDay || this.renderDay)(day, this.props.value, this.state.screen, this.state.cursor, () => this.onChange(day), () => {
            if (!sameDays(day, this.state.cursor)) {
                this.onCursorChange(day);
            }
        });
        const abbreviations = (this.props.renderAbbreviations || this.renderAbbreviations)(i18n$$1, weekStart);
        const header = (this.props.renderHeader || this.renderHeader)(i18n$$1, this.state.screen, this.onScreenChange);
        return (createElement(DatePicker, { matrix: matrix, onKeyDown: e => this.onKeyDown(e, matrix), renderDay: renderDay, abbreviations: abbreviations, header: header }));
    }
}

export { defaultOnKeyDown, SimpleDatePicker, classNames, range, sameDays, deltaDate, getFirstDayOfMonth, getFirstDayOfWeek, getLastDayOfMonth, getLastDayOfWeek, generateMatrix, isInMatrix, abbreviationDays, i18n, DatePicker, Header, Day, Abbreviations };
//# sourceMappingURL=simple_datepicker.es6.js.map
